package InitialSchedules;

import ScheduleUtil.FullTimeEmployeeScheduleArray;
import org.chocosolver.solver.constraints.extension.Tuples;

public class ParametersInitialSchedules {
   
    //General Parameters
    final int shiftWorkPerDay = 3;
    final int daysPerSchedule = 14;
    final int hoursPerWorkPeriod = 4;
    final int workingHoursPaidAtRegularHourlyRatePerSchedule = 80;
    final int workingHoursPaidAtRegularHourlyRatePerShiftWork = 8;
    

    //PartTime Employees Parameters
    final int fixedCostOfPartTimeEmployeesPerSchedule = 50;
    final int regularHourlyRateOfPartTimeEmployees = 12;
    final int overtimeHourlyRateOfPartTimeEmployees = 17;
    final int minWorkingHoursOfPartTimeEmployeesPerSchedule = 32;
    final int maxWorkingHoursOfPartTimeEmployeesPerSchedule = 64;
    final int maxConsecutiveWorkingHoursOfPartTimeEmployeesPerShiftWork = 12;
    final int minConsecutiveNonWorkingHoursBetweenShiftWorksOfPartTimeEmployees = 12;
    
    //FullTime Employees Parameters
    final int fixedCostOfFullTimeEmployeesPerSchedule = 50;
    final int regularHourlyRateOfFullTimeEmployees = 10;
    final int overtimeHourlyRateOfFullTimeEmployees = 15;
    final int workingHoursOfFullTimeEmployeesPerSchedule = 80;
    final int maxWorkingHoursOfFullTimeEmployeesPerSchedule = 120;
    final int maxConsecutiveWorkingHoursOfFullTimeEmployeesPerShiftWork = 12;
    final int minConsecutiveNonWorkingHoursBetweenShiftWorksOfFullTimeEmployees = 12;

    int[] requiredWorkforce;
    int NbFullTimeSolutions;
    int NbPartTimeSolutions;
    Tuples enumerationWorkPeriodsSchedulesOfFullTimeEmployees;
    Tuples enumerationWorkPeriodsSchedulesOfPartTimeEmployees;
    Tuples ValidPartTimeEmployeeShiftTuples;

    public ParametersInitialSchedules() {

        this.setRequiredWorkforce();
        this.enumerationWorkPeriodsSchedulesOfFullTimeEmployees = GenerationPossibleEmployeeSchedules.setWorkPeriodsSchedulesOfFullTimeEmployees(
                                                                            shiftWorkPerDay, getWorkPeriodsPerSchedule(), getWorkPeriodPerDay(), daysPerSchedule);
        this.enumerationWorkPeriodsSchedulesOfPartTimeEmployees = GenerationPossibleEmployeeSchedules.setWorkPeriodsSchedulesOfPartTimeEmployees(
                                                                            getWorkPeriodsPerSchedule(), getWorkPeriodPerDay(), daysPerSchedule);
        this.ValidPartTimeEmployeeShiftTuples = new ValidPartTimeEmployeeShift().makeTuples();

    }

    private void setRequiredWorkforce() {

        this.requiredWorkforce = new int[this.daysPerSchedule * this.getWorkPeriodPerDay()];
        int[] dailyRequiredWorkforce = new int[]{2, 2, 4, 4, 3, 3};
        for (int day = 0; day < this.daysPerSchedule; day++) {
            System.arraycopy(dailyRequiredWorkforce, 0, this.requiredWorkforce, day * this.getWorkPeriodPerDay(), this.getWorkPeriodPerDay());
        }
    }
    
    public void adjustRequiredWorkforce( FullTimeEmployeeScheduleArray fullTimeSchedule) {
        
        for (int workPeriod = 0 ; workPeriod < this.getWorkPeriodsPerSchedule() ; workPeriod++) {
            this.requiredWorkforce[workPeriod] -= fullTimeSchedule.getEmployeesWorkingPerWorkPeriod(workPeriod);
        }
        
    }
    
    public int getTotalWorkedPeriodsInSchedule() {
        int totalWorkedPeriodsInSchedule = 0;
        for (int workPeriod = 0 ; workPeriod < this.getWorkPeriodsPerSchedule() ; workPeriod++) {
            totalWorkedPeriodsInSchedule += this.requiredWorkforce[workPeriod];
        }
        return totalWorkedPeriodsInSchedule;
    }

    public int getRequiredWorkForce(int workPeriod) {
        return this.requiredWorkforce[workPeriod];
    }
    
    public int getDaysPerSchedule() {
        return daysPerSchedule;
    }
    
    public int getWorkPeriodsPerSchedule() {
        return this.getWorkPeriodPerDay() * this.daysPerSchedule;
    }
    
    public int getWorkPeriodPerDay() {
        return (int) 24/this.hoursPerWorkPeriod;
    }
    
    public int  getFixedCostOfPartTimeEmployeesPerSchedule() {
        return this.fixedCostOfPartTimeEmployeesPerSchedule;
    }
    
    public int  getFixedCostOfFullTimeEmployeesPerSchedule() {
        return this.fixedCostOfFullTimeEmployeesPerSchedule;
    }

    public int getWorkingPeriodsPaidAtRegularHourlyRatePerSchedule () {
        return (int) (workingHoursPaidAtRegularHourlyRatePerSchedule / this.hoursPerWorkPeriod);
    }

    public int getWorkingPeriodsPaidAtRegularHourlyRatePerShiftWork () {
        return (int) (workingHoursPaidAtRegularHourlyRatePerShiftWork / this.hoursPerWorkPeriod);
    }

    public int getWorkingPeriodCostOfPartTimeEmployeesPaidAtRegularHourlyRate () {
        return (int) (regularHourlyRateOfPartTimeEmployees * this.hoursPerWorkPeriod);
    }

    public int getWorkingPeriodCostOfFullTimeEmployeesPaidAtRegularHourlyRate () {
        return (int) (regularHourlyRateOfFullTimeEmployees * this.hoursPerWorkPeriod);
    }
    
    public int getMinimumWorkingPeriodCost () {
        return (int) (regularHourlyRateOfFullTimeEmployees * this.hoursPerWorkPeriod);
    }

    public int getWorkingPeriodCostOfPartTimeEmployeesPaidAtOvertimeHourlyRate () {
        return (int) (overtimeHourlyRateOfPartTimeEmployees * this.hoursPerWorkPeriod);
    }

    public int getWorkingPeriodCostOfFullTimeEmployeesPaidAtOvertimeHourlyRate () {
        return (int) (overtimeHourlyRateOfFullTimeEmployees * this.hoursPerWorkPeriod);
    }

    public int getMaxConsecutiveWorkingPeriodsOfFullTimeEmployeesPerShiftWork () {
        return (int) (maxConsecutiveWorkingHoursOfFullTimeEmployeesPerShiftWork / this.hoursPerWorkPeriod);
    }

    public int getMaxConsecutiveWorkingPeriodsOfPartTimeEmployeesPerShiftWork () {
        return (int) (maxConsecutiveWorkingHoursOfPartTimeEmployeesPerShiftWork / this.hoursPerWorkPeriod);
    }

    public int getMinConsecutiveNonWorkingPeriodsBetweenShiftWorksOfPartTimeEmployees () {
        return (int) (minConsecutiveNonWorkingHoursBetweenShiftWorksOfPartTimeEmployees / this.hoursPerWorkPeriod);
    }

    public int getMinConsecutiveNonWorkingPeriodsBetweenShiftWorksOfFullTimeEmployees () {
        return (int) (minConsecutiveNonWorkingHoursBetweenShiftWorksOfFullTimeEmployees / this.hoursPerWorkPeriod);
    }

    public int getMaxWorkingPeriodsOfPartTimeEmployeesPerSchedule () {
        return (int) (this.maxWorkingHoursOfPartTimeEmployeesPerSchedule / this.hoursPerWorkPeriod);
    }

    public int getMaxWorkingPeriodsOfFullTimeEmployeesPerSchedule () {
        return (int) (this.maxWorkingHoursOfFullTimeEmployeesPerSchedule / this.hoursPerWorkPeriod);
    }

    public int getMinWorkingPeriodsOfPartTimeEmployeesPerSchedule () {
        return (int) (this.minWorkingHoursOfPartTimeEmployeesPerSchedule / this.hoursPerWorkPeriod);

    }

    public int getWorkingPeriodsOfFullTimeEmployeesPerSchedule () {
        return (int) (this.workingHoursOfFullTimeEmployeesPerSchedule / this.hoursPerWorkPeriod);
    }

    // A implementer plus tard si l'on veut travailler avec des fichiers texte
    public ParametersInitialSchedules(String fileName) {}

}
